/** \file
 * This file describes the core configuration
 */
#ifndef ARCLIB_CONFIGCORE
#define ARCLIB_CONFIGCORE

#include <iostream>
#include <list>
#include <map>
#include <string>

#include <arc/error.h>

/**
 * The basic type of a configuration variable.
 * The values are represented as a list some options are multivalued.
 */
typedef std::list<std::string> OptionValue;

/**
 * The internal representation of the full configuration. It is a simple
 * simple map from strings to OptionValue, which again is a list of strings.
 */
typedef std::map<std::string, OptionValue> Options;


/**
 * Error configuration class.
 */
class ConfigError : public ARCLibError {

	public:
		/** Constructor for the ConfigError exception. Calls the corresponding
		 *  constructor in ARCLibError.
		 */
		ConfigError(std::string message) : ARCLibError(message) {};
};


/** Core configuration class. */
class Config {

	public:

		/** Get the configuration values from key. */
		void AddConfValue(std::string path, std::string);

		/** Get the configuration values from key. */
		OptionValue ConfValue(std::string path);

		/** Get the first configuration value from key.
		 *  This is meant as a short cut when it is known that the key is
		 *  not multivalued.
		 */
		std::string FirstConfValue(std::string path);

		/** Returns the parsed options. */
		Options& GetOptions();

		/** Returns the number of options. */
		int Size();

	private:
		/** The parsed options. */
		Options content;
};


/** Read configuration from input stream trying all known formats */
Config ReadConfig(std::istream& is);

/** Read configuration from file trying all known formats */
Config ReadConfig(const std::string& filename);


#endif //  ARCLIB_CONFIGCORE
